# -*- coding: utf-8 -*-
"""Unit tests for Hawkular client."""
import json
from urlparse import urlparse

import os
import pytest
from mgmtsystem import hawkular
from mock import patch


def fake_urlopen(c_client, url):
    """
    A stub urlopen() implementation that load json responses from
    the filesystem.
    """
    # Map path from url to a file
    parsed_url = urlparse("{}/{}".format(c_client.api_entry, url))
    resource_file = os.path.normpath("tests/resources/{}.json".format(parsed_url.path))
    # Must return a file-like object
    return json.load(open(resource_file))


@pytest.yield_fixture(scope="function")
def provider():
    """
    A stub urlopen() implementation that load json responses from
    the filesystem.
    """
    hwk = hawkular.Hawkular(
        hostname=os.getenv('HAWKULAR_HOSTNAME', 'localhost'),
        protocol=os.getenv('HAWKULAR_PROTOCOL', 'http'),
        port=os.getenv('HAWKULAR_PORT', 8080),
        username=os.getenv('HAWKULAR_USERNAME', 'jdoe'),
        password=os.getenv('HAWKULAR_PASSWORD', 'password')
    )
    if not os.getenv('HAWKULAR_HOSTNAME'):
        hwk.patcher = patch('mgmtsystem.rest_client.ContainerClient.get_json', fake_urlopen)
        hwk.patcher.start()
    yield hwk
    if not os.getenv('HAWKULAR_HOSTNAME'):
        hwk.patcher.stop()


def test_list_feed(provider):
    """ Checks whether any feed is listed """
    feeds = provider.list_feed()
    assert len(feeds) > 0, "No feeds are listed"
    for feed in feeds:
        assert feed.id
        assert feed.path


def test_list_resource_type(provider):
    """ Checks whether any resource type is listed and has attributes """
    feeds = provider.list_feed()
    for feed in feeds:
        res_types = provider.list_resource_type(feed_id=feed.id)
        for res_type in res_types:
            assert res_type.id
            assert res_type.name
            assert res_type.path
    assert len(res_types) > 0, "No resource type is listed for any of feeds"


def test_list_server(provider):
    """ Checks whether any server is listed and has attributes"""
    servers = provider.list_server()
    for server in servers:
        assert server.id
        assert server.name
        assert server.path
        assert server.data['data_name']
        assert server.data['Hostname']
        assert server.data['Server State']
        assert server.data['Version']
        assert server.data['Product Name']
    assert len(servers) > 0, "No server is listed for any of feeds"


def test_list_server_deployment(provider):
    """ Checks whether any deployment is listed and has attributes """
    deployments = provider.list_server_deployment()
    for deployment in deployments:
        assert deployment.id
        assert deployment.name
        assert deployment.path
    assert len(deployments) > 0, "No deployment is listed for any of feeds"


def test_resource_data(provider):
    """ Checks whether resource data is provided and has attributes """
    found = False
    servers = provider.list_server()
    for server in servers:
        r_data = provider.resource_data(feed_id=server.path.feed, resource_id=server.id)
        if r_data:
            found = True
            assert r_data.name
            assert r_data.path
            assert r_data.value
    assert found, "No resource data is listed for any of servers"


def test_list_server_datasource(provider):
    """ Checks whether any datasource is listed and has attributes """
    found = False
    datasources = provider.list_server_datasource()
    if len(datasources) > 0:
        found = True
    for datasource in datasources:
        assert datasource.id
        assert datasource.name
        assert datasource.path
    assert (found | provider._stats_available['num_datasource'](provider) > 0,
            "No any datasource is listed for any of feeds, but they exists")


def test_path(provider):
    """ Checks whether path returned correctly """
    feeds = provider.list_feed()
    for feed in feeds:
        assert feed.path
        assert feed.path.tenant
        assert feed.path.feed
    servers = provider.list_server()
    for server in servers:
        assert server.path
        assert server.path.tenant
        assert server.path.feed
        assert server.path.resource


def test_num_server(provider):
    """ Checks whether number of servers is returned correct """
    servers_count = 0
    feeds = provider.list_feed()
    for feed in feeds:
        servers_count += len(provider.list_server(feed_id=feed.id))
    num_server = provider._stats_available['num_server'](provider)
    assert num_server == servers_count, "Number of servers is wrong"


def test_num_deployment(provider):
    """ Checks whether number of deployments is returned correct """
    deployments_count = 0
    feeds = provider.list_feed()
    for feed in feeds:
        deployments_count += len(provider.list_server_deployment(feed_id=feed.id))
    num_deployment = provider._stats_available['num_deployment'](provider)
    assert num_deployment == deployments_count, "Number of deployments is wrong"


def test_num_datasource(provider):
    """ Checks whether number of datasources is returned correct """
    datasources_count = 0
    feeds = provider.list_feed()
    for feed in feeds:
        datasources_count += len(provider.list_server_datasource(feed_id=feed.id))
    num_datasource = provider._stats_available['num_datasource'](provider)
    assert num_datasource == datasources_count, "Number of datasources is wrong"


def test_list_event(provider):
    """ Checks whether is any event listed """
    events = provider.list_event()
    assert len(events) > 0, "No events are listed"
    event = events[0]
    assert event.id
    assert event.eventType
    assert event.ctime
    assert event.dataSource
    assert event.dataId
    assert event.category
    assert event.text



from base import MgmtSystemAPIBase
from collections import namedtuple
from rest_client import ContainerClient

import re
import sys

"""
Related yaml structures:

[cfme_data]
management_systems:
    hawkular:
        name: My hawkular
        type: hawkular
        hostname: 10.12.13.14
        port: 8080
        credentials: hawkular
        authenticate: true
        rest_protocol: http

[credentials]
hawkular:
    username: admin
    password: secret
"""

Feed = namedtuple('Feed', ['id', 'name', 'path'])
ResourceType = namedtuple('ResourceType', ['id', 'name', 'path'])
Resource = namedtuple('Resource', ['id', 'name', 'path'])
ResourceData = namedtuple('ResourceData', ['name', 'path', 'value'])
Server = namedtuple('Server', ['id', 'name', 'path', 'data'])
Deployment = namedtuple('Deployment', ['id', 'name', 'path'])
Datasource = namedtuple('Datasource', ['id', 'name', 'path'])
ServerStatus = namedtuple('ServerStatus', ['address', 'version', 'state', 'product', 'host'])
Event = namedtuple('event', ['id', 'eventType', 'ctime', 'dataSource', 'dataId',
                             'category', 'text'])

PATH_NAME_MAPPING = {
    '/t;': 'tenant',
    '/e;': 'environment',
    '/rt;': 'resource_type',
    '/mt;': 'metric_type',
    '/f;': 'feed',
    '/ot;': 'operation_type',
    '/mp;': 'metadata_pack',
    '/r;': 'resource',
    '/d;': 'data',
    '/rl;': 'relationship',
}


class Path(object):
    """Path class

    Path is class to split canonical path to friendly values.\
    If the path has more than one entry for a resource result will be in list
    Example:
        obj_p = Path('/t;28026b36-8fe4-4332-84c8-524e173a68bf\
        /f;88db6b41-09fd-4993-8507-4a98f25c3a6b\
        /r;Local~~/r;Local~%2Fdeployment%3Dhawkular-command-gateway-war.war')
        obj_p.path returns raw path
        obj_p.tenant returns tenant as `28026b36-8fe4-4332-84c8-524e173a68bf`
        obj_p.feed returns feed as `88db6b41-09fd-4993-8507-4a98f25c3a6b`
        obj_p.resource returns as \
        `[u'Local~~', u'Local~%2Fdeployment%3Dhawkular-command-gateway-war.war']`

    Args:
        path:   The canonical path. Example: /t;28026b36-8fe4-4332-84c8-524e173a68bf\
        /f;88db6b41-09fd-4993-8507-4a98f25c3a6b/r;Local~~

    """
    def __init__(self, path):
        self.paths = {'raw': path}
        if self.raw:
            raw_paths = re.split(r'(/\w+;)', self.raw)
            if len(raw_paths) % 2 == 1:
                del raw_paths[0]
            for p_index in range(0, len(raw_paths), 2):
                if PATH_NAME_MAPPING[raw_paths[p_index]] in self.paths:
                    if isinstance(self.paths[PATH_NAME_MAPPING[raw_paths[p_index]]], list):
                        self.paths[PATH_NAME_MAPPING[raw_paths[p_index]]] \
                            .append(raw_paths[p_index + 1])
                    else:
                        v_list = [
                            self.paths[PATH_NAME_MAPPING[raw_paths[p_index]]],
                            raw_paths[p_index + 1]
                        ]
                        self.paths.update({PATH_NAME_MAPPING[raw_paths[p_index]]: v_list})
                else:
                    self.paths.update(
                        {PATH_NAME_MAPPING[raw_paths[p_index]]: raw_paths[p_index + 1]})

    def __repr__(self):
        return self.paths['raw'] if 'raw' in self.paths else None

    def __getattr__(self, name):
        return self.paths[name] if name in self.paths else None


class Hawkular(MgmtSystemAPIBase):
    """Hawkular management system

    Hawkular REST API method calls.
    Will be used by cfme_tests project to verify Hawkular content shown in CFME UI

    Args:
        hostname: The Hawkular hostname.
        protocol: Hawkular REST API protocol. Default value: 'http'
        port: Hawkular REST API port on provided host. Default value: '8080'.
        entry: Hawkular REST API entry point URI. Default value: 'hawkular/inventory'
        username: The username to connect with.
        password: The password to connect with.

    """

    _stats_available = {
        'num_server': lambda self: len(self.list_server()),
        'num_deployment': lambda self: len(self.list_server_deployment()),
        'num_datasource': lambda self: len(self.list_server_datasource()),
    }

    def __init__(self,
            hostname, protocol="http", port=8080, **kwargs):
        super(Hawkular, self).__init__(kwargs)
        self.hostname = hostname
        self.username = kwargs.get('username', '')
        self.password = kwargs.get('password', '')
        self.auth = self.username, self.password
        self.inv_api = ContainerClient(hostname, self.auth, protocol, port, "hawkular/inventory")
        self.alerts_api = ContainerClient(hostname, self.auth, protocol, port, "hawkular/alerts")

    def info(self):
        raise NotImplementedError('info not implemented.')

    def clone_vm(self, source_name, vm_name):
        raise NotImplementedError('clone_vm not implemented.')

    def create_vm(self, vm_name):
        raise NotImplementedError('create_vm not implemented.')

    def current_ip_address(self, vm_name):
        raise NotImplementedError('current_ip_address not implemented.')

    def delete_vm(self, vm_name):
        raise NotImplementedError('delete_vm not implemented.')

    def deploy_template(self, template, *args, **kwargs):
        raise NotImplementedError('deploy_template not implemented.')

    def disconnect(self):
        pass

    def does_vm_exist(self, name):
        raise NotImplementedError('does_vm_exist not implemented.')

    def get_ip_address(self, vm_name):
        raise NotImplementedError('get_ip_address not implemented.')

    def is_vm_running(self, vm_name):
        raise NotImplementedError('is_vm_running not implemented.')

    def is_vm_stopped(self, vm_name):
        raise NotImplementedError('is_vm_stopped not implemented.')

    def is_vm_suspended(self, vm_name):
        raise NotImplementedError('is_vm_suspended not implemented.')

    def list_flavor(self):
        raise NotImplementedError('list_flavor not implemented.')

    def list_template(self):
        raise NotImplementedError('list_template not implemented.')

    def list_vm(self, **kwargs):
        raise NotImplementedError('list_vm not implemented.')

    def remove_host_from_cluster(self, hostname):
        raise NotImplementedError('remove_host_from_cluster not implemented.')

    def restart_vm(self, vm_name):
        raise NotImplementedError('restart_vm not implemented.')

    def start_vm(self, vm_name):
        raise NotImplementedError('start_vm not implemented.')

    def stop_vm(self, vm_name):
        raise NotImplementedError('stop_vm not implemented.')

    def suspend_vm(self, vm_name):
        raise NotImplementedError('restart_vm not implemented.')

    def vm_status(self, vm_name):
        raise NotImplementedError('vm_status not implemented.')

    def wait_vm_running(self, vm_name, num_sec):
        raise NotImplementedError('wait_vm_running not implemented.')

    def wait_vm_stopped(self, vm_name, num_sec):
        raise NotImplementedError('wait_vm_stopped not implemented.')

    def wait_vm_suspended(self, vm_name, num_sec):
        raise NotImplementedError('wait_vm_suspended not implemented.')

    def list_server_deployment(self, **kwargs):
        """Returns list of server deployments. Possible filters: `feed_id`"""
        resources = self.list_resource(type_id='Deployment',
                                       feed_id=kwargs['feed_id'] if 'feed_id' in kwargs else None)
        deployments = []
        if resources:
            for resource in resources:
                deployments.append(Deployment(resource.id, resource.name, resource.path))
        return deployments

    def list_server(self, **kwargs):
        """Returns list of middleware servers. Possible filters: `feed_id`"""
        resources = self.list_resource(type_id='WildFly Server',
                                       feed_id=kwargs['feed_id'] if 'feed_id' in kwargs else None)
        servers = []
        if resources:
            for resource in resources:
                resource_data = self.resource_data(
                    feed_id=resource.path.feed, resource_id=resource.id)
                server_data = {'data_name': resource_data.name}
                server_data.update(resource_data.value)
                servers.append(Server(resource.id, resource.name, resource.path, server_data))
        return servers

    def list_resource(self, **kwargs):
        """Returns list of resources. Possible filters: `feed_id`, `type_id`"""
        feed_id = kwargs['feed_id'] if 'feed_id' in kwargs else None
        if not feed_id:
            resources = []
            feeds = self.list_feed()
            for feed in feeds:
                resources = \
                    resources + self._list_resource(type_id=kwargs['type_id'], feed_id=feed.id)
            return resources
        else:
            return self._list_resource(type_id=kwargs['type_id'], feed_id=feed_id)

    def _list_resource(self, **kwargs):
        """Returns list of resources by provided `type_id` and `feed_id`"""
        if not kwargs or 'feed_id' not in kwargs:
            raise KeyError('Variable "feed_id" is a mandatory field!')
        entities = []
        if kwargs['type_id']:
            entities_j = self.inv_api.get_json('feeds/{}/resourceTypes/{}/resources'
                                               .format(kwargs['feed_id'], kwargs['type_id']))
        else:
            entities_j = self.inv_api.get_json('feeds/{}/resources'.format(kwargs['feed_id']))
        if entities_j:
            for entity_j in entities_j:
                entities.append(Resource(entity_j['id'], entity_j['name'], Path(entity_j['path'])))
        return entities

    def resource_data(self, **kwargs):
        """Returns the data/configuration information about resource by provided\
         `feed_id` and `resource_id`."""
        if not kwargs or 'feed_id' not in kwargs or 'resource_id' not in kwargs:
            raise KeyError('Variable "feed_id" and "resource_id" are mandatory field!')
        entity_j = self.inv_api.get_json('feeds/{}/resources/{}/data'
                                     .format(kwargs['feed_id'], kwargs['resource_id']))
        if entity_j:
            return ResourceData(entity_j['name'], Path(entity_j['path']), entity_j['value'])
        return None

    def list_feed(self):
        """Returns list of feeds"""
        entities = []
        entities_j = self.inv_api.get_json('feeds')
        if entities_j:
            for entity_j in entities_j:
                entity = Feed(entity_j['id'],
                              entity_j['name'] if 'name' in entity_j else None,
                              Path(entity_j['path']))
                entities.append(entity)
        return entities

    def list_resource_type(self, **kwargs):
        """Returns list of resource types by provided `feed_id`"""
        if not kwargs or 'feed_id' not in kwargs:
            raise KeyError('Variable "feed_id" is a mandatory field!')
        entities = []
        entities_j = self.inv_api.get_json('feeds/{}/resourceTypes'.format(kwargs['feed_id']))
        if entities_j:
            for entity_j in entities_j:
                entity = ResourceType(entity_j['id'], entity_j['name'], entity_j['path'])
                entities.append(entity)
        return entities

    def list_event(self, start_time=0, end_time=sys.maxsize):
        """Returns the list of events filtered by provided start time and end time.
        Or lists all events if no argument provided.
        This information is wrapped into Event."""
        entities = []
        entities_j = self.alerts_api.get_json('events?startTime={}&endTime={}'
                                     .format(start_time, end_time))
        if entities_j:
            for entity_j in entities_j:
                entity = Event(entity_j['id'], entity_j['eventType'], entity_j['ctime'],
                               entity_j['dataSource'], entity_j['dataId'],
                               entity_j['category'], entity_j['text'])
                entities.append(entity)
        return entities

    def list_server_datasource(self, **kwargs):
        """Returns list of datasources. Possible filters: `feed_id`"""
        resources = self.list_resource(type_id='Datasource',
                                       feed_id=kwargs['feed_id'] if 'feed_id' in kwargs else None)
        datasources = []
        if resources:
            for resource in resources:
                datasources.append(Datasource(resource.id, resource.name, resource.path))
        return datasources
